//Import from mousegestures.rdf

var importer = {
  RDF : Components.classes["@mozilla.org/rdf/rdf-service;1"]
                  .getService(Components.interfaces.nsIRDFService),

  NS : "http://optimoz.mozdev.org/gestures-rdf#",

  actTemp : new Array(),

  initImport : function() {
    var retVal = false;
    try {
      var wTypeArray = new Array("browser", "viewsource", "window", "messenger", "chatzilla");

      var dirService   = Components.classes['@mozilla.org/file/directory_service;1']
                                   .getService(Components.interfaces.nsIProperties);
      var mappingsFile = dirService.get('ProfD', Components.interfaces.nsILocalFile);
      mappingsFile.append("mousegestures.rdf");

      var ioService  = Components.classes["@mozilla.org/network/io-service;1"]
                                 .getService(Components.interfaces.nsIIOService);
      var mgFileSpec = ioService.newFileURI(mappingsFile).spec

      var dsource = this.RDF.GetDataSourceBlocking(mgFileSpec);
      // if dsource is corrupted, we will not receive the error,
      // although it is in the console. so let us test...
      var lastTest = dsource.GetAllResources();

      if (!lastTest.hasMoreElements())
        throw("MozGest: Stopping import! Something is wrong with mousegestures.rdf\n");

      for (var x = 0; x < wTypeArray.length; x++)
        this.importRDFGestures(dsource, wTypeArray[x]);
    }
    catch (e) {
      retVal = e;
    }

    return retVal;
  },

  importRDFGestures : function(dsource, winType) {
    var mappingsContainer =  Components.classes['@mozilla.org/rdf/container;1'].createInstance()
                             .QueryInterface(Components.interfaces.nsIRDFContainer);

    var rootnode = this.RDF.GetResource("urn:mozgest:mappings:" + winType);

    try {
      mappingsContainer.Init(dsource, rootnode);
    }
    catch (e) {
      // no existing mappings
      return;
    }

    var enumerator = mappingsContainer.GetElements();

    if (!enumerator.hasMoreElements())
      return;

    this.actTemp[winType] = new Array();

    while (enumerator.hasMoreElements()) {
      var mapping = enumerator.getNext()
                    .QueryInterface(Components.interfaces.nsIRDFResource);

      var code = this.getProperty(dsource, mapping, "code");

      if (code == "UUUUUU")
        continue;

      var appearance = this.getProperty(dsource, mapping, "appearance");
      var func = this.getProperty(dsource, mapping, "function");
      var custom = this.getProperty(dsource, mapping, "custom");
      var bmOpenIn = this.getProperty(dsource, mapping, "bmOpenIn");
      var count = Math.abs(this.getProperty(dsource, mapping, "count")) || 0;
      var type = null;
      var name = null;

      if (custom) {
        type = "1";
        func = custom;
        name = this.getProperty(dsource, mapping, "name");
      }
      if (bmOpenIn)
        type = "2" + bmOpenIn;

      this.actTemp[winType][code] = {type:type, name:name, func:func, count:count, appearance:appearance};
    }
    return;
  },

  getProperty : function (dsource, obj, propName) {
    var literal = dsource.GetTarget(obj, this.RDF.GetResource(this.NS + propName), true);
    if (literal)
      return literal.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
    else
      return null;
  }
}