/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mouse Gestures for Mozilla.
 *
 * The Initial Developer of the Original Code is Jens Bannmann.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s) (alphabetical order):
 *  Jens Bannmann <jens.b@web.de>
 *  Jochen <bugs@krickelkrackel.de>
 *  Neil <neil@parkwaycc.co.uk>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var dialogMode;
var actTemp, wType, docLoc;
var curMapping = false;

var gestureType;
var gestureInput;
var codeBox;
var appBox;
var wheelBtn;
var wheelDir;
var functionType;
var functionsTree;
var ftChildren;
var mappingName;

var startType, startCode, startAppearance, startName, startFunc, startCount;

var bmMappingName;
var bmTree;
var bmSelected;
var bmOpenIn;
var bmDeleted = false;
var bmInfo;
var bmImage;
var mgRow1, mgRow2, mgRow3;
var bmp_attention = "chrome://mozgest/skin/attention.png";
var bmp_placeholder = "chrome://mozgest/skin/placeholder.png";


function initEditor() {
  gestureType   = document.getElementById("gestureType");
  gestureInput  = document.getElementById("gestureInput");
  codeBox       = document.getElementById("codeBox");
  appBox        = document.getElementById("appearanceBox");
  wheelBtn      = document.getElementById("wheelRockerButton");
  wheelDir      = document.getElementById("wheelDirection");
  functionType  = document.getElementById("functionType");
  functionsTree = document.getElementById("functionsTree");
  ftChildren    = document.getElementById("ftChildren");
  mappingName   = document.getElementById("mappingName");
  bmMappingName = document.getElementById("bmMappingName");
  bmInfo        = document.getElementById("bmInfo");
  bmImage       = document.getElementById("bmImage");
  bmOpenIn      = document.getElementById("bmOpenIn");
  mgRow1        = document.getElementById("mgRow1");
  mgRow2        = mgRow1.nextSibling;
  mgRow3        = mgRow2.nextSibling;

  bmSelected    = "";

  if (mgAppInfo.name == "Firefox")
    bmOpenIn.lastChild.hidden = false;

  // get passed arguments
  // [0] == dialogMode (Edit, New or Import)
  // [1] == wType (window, browser, etc.)
  // [2] == gesture code
  dialogMode = window.arguments[0];
  wType      = window.arguments[1];
  curMapping = window.arguments[2];

  initDirectionCodes();

  var docTitle = mgGetString(dialogMode + "MappingWindow")
                 + " - " + document.documentElement.getAttribute("mg" + wType);
  document.title = docTitle;


  // fill function list
  var curFuncts = _mgMS.functions[wType];

  for (func in curFuncts)
    mgCommon.addTreeRow(ftChildren, new Array(func, mgGetString(func)));

  if (wType != "window") {
    curFuncts = _mgMS.functions["window"];

    for (func in curFuncts)
      mgCommon.addTreeRow(ftChildren, new Array(func,mgGetString(func)), "mozgestGeneric");
  }

  // prepare for bookmark mappings
  if (wType != "browser" || dialogMode == "Import")
    mgBMService.enabled = mgBMService.plEnabled = false;

  if (mgBMService.enabled || mgBMService.plEnabled) {
    document.getElementById("miBookmark").removeAttribute("hidden");
    bmOpenIn.selectedIndex = 0;
    bmTree = (mgBMService.enabled) ? document.getElementById("mgBookmarksTree") : document.getElementById("mgPlacesTree");
    bmTree.removeAttribute("hidden");
    (mgBMService.enabled) ? updateInfoBox(true) : plUpdateInfoBox(true);;
  }

  actTemp = _mgMS.actTemp;

  if (dialogMode == "New") {
    if (_mgMS.curBookmark)  {
      bmSelected = _mgMS.curBookmark;
      _mgMS.curBookmark = null;
      bmMappingName.value = _mgMS.curBookmarkName;

      if (mgBMService.enabled)
        bmInit();
      else if (mgBMService.plEnabled)
        setTimeout(plInit, 0);

      functionType.selectedIndex = 2;
      updateFunctionDeck();
    }
    return;
  }

  if (dialogMode == "Edit") {
    startCode = curMapping;
    if (actTemp[wType][curMapping].appearance) {
      startAppearance = actTemp[wType][curMapping].appearance;
      appBox.value = mgMappingLocalizer.localize(actTemp[wType][curMapping].appearance);
    }
    else
      startAppearance = "";

    if (actTemp[wType][curMapping].name)
      startName = actTemp[wType][curMapping].name;

    startFunc = actTemp[wType][curMapping].func;
    startCount = actTemp[wType][curMapping].count;

    setGestureType();

    // if it is a custom gesture fill the fields
    if (actTemp[wType][curMapping].name) {
      mappingName.value = decodeURIComponent(actTemp[wType][curMapping].name);

      try {
        document.getElementById("custom").value = decodeURIComponent(actTemp[wType][curMapping].func);
      }
      catch (e) {}
    }

    // if it is a bookmark gesture...
    var fType = startType = actTemp[wType][curMapping].type;
    if (!fType)
      fType = startType = "";

    if (fType.toString().indexOf("2") == 0) {
      bmSelected = actTemp[wType][curMapping].func;
      bmMappingName.value = decodeURIComponent(actTemp[wType][curMapping].name);
      bmOpenIn.selectedIndex = fType.toString().substring(1, 2);

      if (mgAppInfo.name != "Firefox" && bmOpenIn.selectedIndex == 4)
        bmOpenIn.selectedIndex = 0;

      bmOpenIn.disabled = false;

      mappingName.value = document.getElementById("custom").value = "";

      if (mgBMService.enabled)
        bmInit();
      else if (mgBMService.plEnabled)
        setTimeout(plInit, 0);
    }

    // try to to select the function
    try {
      for (var x = 0; x < ftChildren.childNodes.length; x++) {
        var funcName = functionsTree.view.getCellText(x, functionsTree.columns["ftFuncCol"]);

        if (funcName == actTemp[wType][curMapping].func) {
          functionsTree.view.selection.select(x);
          break;
        }
      }
    }
    catch (e) {}

    var selFuncRow;
    try {
      if (actTemp[wType][curMapping].type)
        selFuncRow = actTemp[wType][curMapping].type.toString().substring(0,1);
      else
        selFuncRow = 0;

      functionType.selectedIndex = selFuncRow;
    }
    catch (e) {}
  }

  if (dialogMode == "Import") {
    setGestureType();
    importGesture();
  }

  updateTypeDeck();
  updateFunctionDeck();
}

function importGesture() {
  docLoc = mgBMService.makeURI(_mgMS.importLocation);
  mgPref.QueryInterface(Components.interfaces.nsIPrefBranchInternal)
        .addObserver("", mgPermissionObserver, false);
  checkPermissions();

  if (_mgMS.importAppearance)
    appBox.value = mgMappingLocalizer.localize(_mgMS.importAppearance);

  if (_mgMS.importName)
    mappingName.value = _mgMS.importName;

  // if _mgMS.importFunc is null than it is a custom gesture!
  if (!_mgMS.importFunc) {
    var custom = document.getElementById("custom");
    custom.value = _mgMS.importCustom;
    functionType.selectedIndex = 1;
  }
  else {
    try {
      for (var x = 0; x < ftChildren.childNodes.length; x++) {
        var funcName = functionsTree.view.getCellText(x, functionsTree.columns["ftFuncCol"]);
        if (funcName == _mgMS.importFunc) {
          functionsTree.view.selection.select(x);
          break;
        }
      }
    }
    catch (e) {}
  }
}

function bmInit() {
  try {
    var sel = bmTree.view.getIndexOfResource(mgBMService.RDFS.GetResource(bmSelected));
    bmTree.builderView.selection.select(sel);
  }
  catch (e) {}

  var elt = mgBMService.RDFS.GetResource(bmSelected);
  var target = mgBMService.Book.GetTarget(elt, mgBMService.Href, true);

  // mapping with an unknown(deleted) bookmark
  if (!target)
    bmOpenIn.disabled = bmDeleted = true;

  updateInfoBox();

  // new gesture from bookmark properties -> prefill mappingname
  if (dialogMode == "New" && target) {
    var bmName = mgBMService.Book.GetTarget(elt, mgBMService.Name, true);
    if (bmName)
      bmMappingName.value = bmName.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
  }
}

function plInit() {
  var target = false;
  try {
    target = mgBMService.plBook.getBookmarkURI(bmSelected);
  }
  catch (e) {}

  // mapping with an unknown(deleted) bookmark
  if (!target)
    bmOpenIn.disabled = bmDeleted = true;

  plUpdateInfoBox();

  // new gesture from bookmark properties -> prefill mappingname
  if (dialogMode == "New" && target) {
    var bmName = mgBMService.plBook.getItemTitle(bmSelected);
    if (bmName != "")
      bmMappingName.value = bmName;
    else
      bmMappingName.value = mgGetString("noName");
  }
}

function bmSelect(e) {
  if (e && e.type == "dblclick") {
    if (!bmOpenIn.disabled)
      bmMappingName.value = bmInfo.value;

    return;
  }

  var elt = bmTree.view.getResourceAtIndex(e.target.currentIndex);
  var target = mgBMService.Book.GetTarget(elt, mgBMService.Href, true)

  // check for livemarks
  try {
    var check = mgBMService.Book.GetTarget(elt, mgBMService.Type, true);
    check = check.QueryInterface(Components.interfaces.nsIRDFResource).Value.toLowerCase();
    if (check == "http://home.netscape.com/nc-rdf#livemark")
      target = false;
    else {
      var eltParent = bmTree.view.getResourceAtIndex(
                      bmTree.view.getParentIndex(e.target.currentIndex));
      check = mgBMService.Book.GetTarget(eltParent, mgBMService.Type, true);
      check = check.QueryInterface(Components.interfaces.nsIRDFResource).Value.toLowerCase();

      if (check == "http://home.netscape.com/nc-rdf#livemark")
        target = false;
    }
  }
  catch (e) {}

  bmOpenIn.disabled = !target;
  if (target)
    bmSelected = elt.QueryInterface(Components.interfaces.nsIRDFNode).Value;
  else
    bmSelected = "";

  updateInfoBox();
}

function plSelect(ctrl, e) {
  if (e && e.type == "dblclick") {
    if (!bmOpenIn.disabled)
      bmMappingName.value = bmInfo.value;

    return;
  }

  var target = bmTree.selectedNode;

  if (target.type != 0)
    target = false;
  else {
     if (target.parent && target.parent.remoteContainerType &&
         target.parent.remoteContainerType == "@mozilla.org/browser/livemark-service;2")
       target = false;
  }

  bmOpenIn.disabled = !target;

  if (target)
    bmSelected = target.itemId;
  else
    bmSelected = "";

  plUpdateInfoBox();
}

function showTooltip(e, target) {
  if (e.type == "click" && e.button == 0) {
    target.removeAttribute('tooltip');
    document.getElementById('mgToolTip')
            .showPopup(document.documentElement, e.screenX,
                       e.screenY, 'tooltip', null, null);
  }
  if (e.type == "mouseout" && !target.hasAttribute('tooltip')) {
    document.getElementById('mgToolTip').hidePopup();
    target.setAttribute('tooltip', 'mgToolTip');
  }
}

function resetInfoBox(emptyNew, bmHref) {
  if (!bmHref) {
    mgRow1.hidden = mgRow2.hidden = mgRow3.hidden = true;

    if (emptyNew) {
      bmInfo.value =  mgRow2.value = mgGetString("chooseBM");
      mgRow2.hidden = false;
      bmOpenIn.disabled = true;
      return false;
    }

    if (bmSelected == "" || bmDeleted) {
      mgRow2.value = mgGetString("chooseBM");
      bmImage.src = bmp_attention;

      if (bmDeleted)
        bmInfo.value = mgRow1.value = mgGetString("unknownBookmark");
      else
        bmInfo.value = mgRow1.value = mgGetString("notValid");

      mgRow1.hidden = mgRow2.hidden = false;
      bmDeleted = false;
      return false;
    }
  }

  if (bmHref) {
    bmOpenIn.disabled = false;
    for (var i = 1; i < bmOpenIn.childNodes.length; i++)
      bmOpenIn.childNodes[i].disabled = false;

    if (bmHref.scheme == "javascript") {
      for (i = 2; i < bmOpenIn.childNodes.length; i++)
        bmOpenIn.childNodes[i].disabled = true;

      bmOpenIn.selectedIndex = 0;
      mgRow2.value = mgGetString("bmBookmarklet");
    }
    else if (bmHref.spec.indexOf("%s") != -1)
      mgRow2.value = mgGetString("bmSearch");
    else
      mgRow2.value = mgGetString("bmBookmark");

    //var bmPost = mgBMService.Book.GetTarget(mgBMService.RDFS.GetResource(bmSelected),
    //                                      mgBMService.Post, true);

    //if (bmPost) {
    //  bmPost = unescape(bmPost.QueryInterface(Components.interfaces.nsIRDFLiteral).Value);
    //  if (bmPost.indexOf("%s") != -1)
    //    mgRow2.value = mgGetString("bmSearch");
    //}

    mgRow2.value = mgGetString("bmType") + mgRow2.value;
    mgRow3.value = mgGetString("bmHref") + bmHref.spec;
    mgRow2.hidden = mgRow3.hidden = false;
  }
  return true;
}

function updateInfoBox(emptyNew) {
  if (!resetInfoBox(emptyNew))
    return;

  var bmName = mgBMService.Book.GetTarget(mgBMService.RDFS.GetResource(bmSelected),
                                          mgBMService.Name, true);
  if (bmName)
    bmName = bmName.QueryInterface(Components.interfaces.nsIRDFLiteral).Value
  else
    bmName = mgGetString("noName");

  bmInfo.value = mgRow1.value = bmName;
  mgRow1.value = mgGetString("bmName") + mgRow1.value;
  mgRow1.hidden = false;

  var bmHref = mgBMService.Book.GetTarget(mgBMService.RDFS.GetResource(bmSelected),
                                          mgBMService.Href, true);

  if (bmHref) {
    bmHref = mgBMService.makeURI(bmHref
             .QueryInterface(Components.interfaces.nsIRDFLiteral).Value);

    resetInfoBox(null, bmHref);
  }

  var bmIcon = mgBMService.Book.GetTarget(mgBMService.RDFS.GetResource(bmSelected),
                                          mgBMService.Icon, true);
  if (bmIcon)
    bmImage.src = bmIcon.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
  else
    bmImage.src = bmp_placeholder;
}

function plUpdateInfoBox(emptyNew) {
  if (!resetInfoBox(emptyNew))
    return;

  var bmName = mgBMService.plBook.getItemTitle(bmSelected);
  if (bmName == "")
    bmName = mgGetString("noName");

  bmInfo.value = mgRow1.value = bmName;
  mgRow1.value = mgGetString("bmName") + mgRow1.value;
  mgRow1.hidden = false;

  var bmHref = mgBMService.plBook.getBookmarkURI(bmSelected);

  if (bmHref)
    resetInfoBox(null, bmHref);

  var options = mgBMService.plHist.getNewQueryOptions();
  var query = mgBMService.plHist.getNewQuery();
  var folderNode = mgBMService.plBook.getFolderIdForItem(bmSelected);

  query.setFolders([mgBMService.plBook.getFolderIdForItem(bmSelected)], 1);

  var result = mgBMService.plHist.executeQuery(query, options);
  document.xxxResult = result;

  result.root.containerOpen = true;
  for (var i = 0; i < result.root.childCount; i ++) {
     var node = result.root.getChild(i);
     if (node.itemId == bmSelected) {
       document.xxxNode = node;

       if (node.icon)
         bmImage.src = node.icon.spec;
       else
         bmImage.src = bmp_placeholder;

       break;
     }
  }
  result.root.containerOpen = false;
}

function setGestureType() {
  if (curMapping.substr(0, 1) == ":") {
    var a = curMapping.substr(1, 1);
    var b = curMapping.substr(2, 1);

    if (!isNaN(b)) {
      gestureType.selectedIndex = 1;
      document.getElementById("rocker1").value = a;
      document.getElementById("rocker2").value = b;
    }
    else {
      gestureType.selectedIndex = 2;
      wheelBtn.value = a;
      wheelDir.selectedIndex = (b == "+") ? 1 : 0;
    }
  }
  else
    codeBox.value = mgMappingLocalizer.localize(curMapping);
}

function updateTypeDeck() {
  document.getElementById("gestureTypeDeck")
          .selectedIndex = gestureType.selectedIndex;
}

function updateFunctionDeck() {
  document.getElementById("functionDeck")
          .selectedIndex = functionType.selectedIndex;
}

function updateSecondRocker() {
  var first = document.getElementById("rocker1").value;

  for (var i=0; i < 3; i++) {
    var si = document.getElementById("second"+i);
    if (i==first)
      si.setAttribute("disabled", true);
    else
      si.removeAttribute("disabled");
  }
  var r2 = document.getElementById("rocker2");
  if (r2.value == first && first != -1)
    r2.value = -1;
}

function checkPermissions() {

  var acceptButton = document.documentElement._buttons.accept;
  var permButton = document.getElementById("permButton");
  var permButtonLabel = document.getElementById("permButtonLabel");
  permButton.hidden = permButtonLabel.hidden = false;
  permButtonLabel.value = mgGetString("permButtonLabel1");
  acceptButton.disabled = true;

  var allowedHosts = mgPref.getCharPref("importFrom").split("|");
  var allow = (docLoc.scheme == "file") ? true : false;

  if (!allow) {
   for (var x = 0; x < allowedHosts.length; x++) {
     if (allowedHosts[x] == docLoc.host) {
       allow = true;
       break;
     }
   }
  }

  if (allow) {
    acceptButton.disabled = false;
    permButtonLabel.value = mgGetString("permButtonLabel0");
  }
}

var mgPermissionObserver = {
  observe: function(subject, topic, data) {
    if (topic.indexOf("nsPref") == 0) {
      if (data == "importFrom")
        checkPermissions();
    }
  }
}

function removeObserver() {
  try {
    mgPref.QueryInterface(Components.interfaces.nsIPrefBranchInternal)
          .removeObserver("", mgPermissionObserver);
  }
  catch (e) {}
}

function saveMapping() {
  var code = mgMappingLocalizer.deLocalize(codeBox.value);
  var appearance = mgMappingLocalizer.deLocalize(appBox.value);

  if (gestureType.selectedIndex == 0) {
    if (code.substring(0,1) == "*") {
      if (code.length < 3 || code.length > 4) {
        alert(mgGetString("enterGesture"));
        return false;
      }
    }
    if (!code.match(/^\*{0,1}[1379DLUR]+$/)
        || mgCommon.compactCode(code) != code) {
      alert(mgGetString("enterGesture"));
      return false;
    }

    if (appearance != "") {
      if (mgCommon.compactCode(appearance) != code) {
        alert(mgGetString("invalidAppearance"));
        return false;
      }
    }
  }
  else if (gestureType.selectedIndex == 1) {
    var a = document.getElementById("rocker1").value;
    var b = document.getElementById("rocker2").value;
    if (a == -1 || b == -1) {
      alert(mgGetString("chooseRockerButtons"));
      return false;
    }
    code = ":" + a + b;
  }
  else if (gestureType.selectedIndex == 2) {
    var wb = wheelBtn.value;
    var wd = wheelDir.selectedIndex;
    if (wd == -1 || wb == -1) {
      alert(mgGetString("chooseWheelDirBtn"));
      return false;
    }
    code = ":" +  wb + ((wd == 1) ? "+" : "-");
  }

  var type   = functionType.selectedIndex;
  var name   = null;
  var func   = null;
  var custom = null;
  var count  = 0;

  if (type == 1) {  // custom gesture
    name = mappingName.value;

    if (name == "") {
      alert(mgGetString("enterMappingName"));
      return false;
    }

    custom = document.getElementById("custom").value;
    if (custom == "") {
       alert(mgGetString("chooseFunction"));
       return false;
    }

    name = encodeURIComponent(name);
    func = encodeURIComponent(custom);
  }
  else if (type == 2) {  // bookmark gesture
    name = bmMappingName.value;

    if (name == "") {
      alert(mgGetString("enterMappingName"));
      return false;
    }

    if (bmSelected == "" || bmOpenIn.disabled) {
      alert(mgGetString("chooseBM"));
      return false;
    }

    name = encodeURIComponent(name);
    func = bmSelected;
    type = type.toString() + bmOpenIn.selectedIndex.toString();
  }
  else {  // normal gesture
    var row = functionsTree.contentView.selection.currentIndex;
    if (row == -1) {
      alert(mgGetString("chooseFunction"));
      return false;
    }

    func = functionsTree.view.getCellText(row, functionsTree.columns["ftFuncCol"]);
  }

  var conf = true;

  if (code in actTemp[wType] && code != startCode) {
    var msg = mgGetString("overwriteWarning");
    var xName;

    if (actTemp[wType][code].name)
      xName = decodeURIComponent(actTemp[wType][code].name);
    else
      xName = mgGetString(actTemp[wType][code].func);

    msg = msg.replace(/%MAPPINGNAME%/, xName);
    var dc = mgMappingLocalizer.localize(code);
    msg = msg.replace(/%CODE%/, dc);
    conf = confirm(msg);
  }

  if (conf) {
    if (dialogMode == "Edit") {
      // let us check if there were changes
      if (type != startType || code != startCode ||
          name != startName || func != startFunc ||
          appearance != startAppearance) {
        // if code was changed, we have to remove the original gesture
        if (code != startCode) {
          //rebuild actTemp without startCode
          var temp = actTemp[wType];
          actTemp[wType] = new Array();

          for (mapping in temp) {
            if (mapping == startCode)
              continue;

            actTemp[wType][mapping] = temp[mapping];
          }
          // if startCode was a default gesture: add an entry to defTemp
          if (startCode in _mgMS.defaultMappings[wType])
            _mgMS.defTemp[wType][startCode] = _mgMS.defaultMappings[wType][startCode];
        }
        else
          count = startCount;
      }
      else
        return true;
    }

    // if code is a default gesture: add an entry to defTemp
    if (code in _mgMS.defaultMappings[wType])
      _mgMS.defTemp[wType][code] = _mgMS.defaultMappings[wType][code];

    // add the mapping to actTemp
    _mgMS.addMapping(wType, code, appearance, type, name, func, count);
  }
  else
    return false;

  return true;
}

function recognizeGesture() {
  window.openDialog("chrome://mozgest/content/pref/recognize.xul",
                    "mozgest:recognize", "chrome,centerscreen,modal");
}

function receiveGesture(code) {
  codeBox.value = mgMappingLocalizer.localize(code);
}